#include <math.h>
#include "../src_f/f77-uscore.h"

/* @20i@ */
/* Function: rayleigh */
/* Description: */
/* Calculates the Rayleigh scattering cross section as a function of */
/* wavelength. Uses the formula due to M. Nicolet, 'On the molecular */
/* scattering in the terrestrial atmosphere: An empirical formula for */
/* its calculation in the homosphere', Planet. Space Sci., vol. 32, */
/* pp. 1467-1468, 1984. */

/* Parameters: */
/*    REAL(*) lambda: Wavelengths (nm) at which the Rayleigh scattering */
/*                    cross section  is to be calculated. (input) */
/*    INTEGER n_lambda: Number of wavelengths lambda. (input) */
/*    INTEGER error:  Error flag. Not in use. */
/*    REAL(*) crs:    The Rayleigh cross section as a function of lambda. */
/*                    (output) */

/* Return value: */
/* Example: */
/* See @file{trans.f}. */
/* Files: */
/*@file{consts.int}  must be present in the same directory as @file{rayleigh.f
}.*/
/* Known bugs: */
/* Author:  Arve Kylling */

/* @i20@ */

int F77_FCN (rayleigh) (float *crs, int *error, float *lambda, int *n_lambda)
{
  
  float x, lambda_um;
  int iv;
  float from_nm_um = 1./1000.;
  float ray_const = 4.02e-28;

  error = 0;
  for (iv = 0; iv <= *n_lambda; ++iv) {
    lambda_um = lambda[iv] * from_nm_um;
    if (lambda_um > .55) {
      x = .04;
    } else {
      x = lambda_um * .389 + .09426 / lambda_um - .3228;
    }
    crs[iv] = ray_const / pow(lambda_um, x+4.);
  }
  return 0;
}


