/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*----------------------------------------------------------------------*/
/************************************************************************/

#ifndef __ck_h
#define __ck_h

#include <stdio.h>
#include "uvspec.h"

/* Kato et al. [1999]           */
/* components: CO2, O3, O2, H2O */
#define KATO_COMPONENTS 4

#define KATO_CO2  1
#define KATO_O3   2
#define KATO_O2   3
#define KATO_H2O  4

#define KATO_MAXINT         20
#define KATO_MAXINT_CO2     18
#define KATO_MAXINT_O3      14
#define KATO_MAXINT_O2       6
#define KATO_MAXINT_H2O     11


/* Fu and Liou [1992/93]               */
/* components: CO2, O3, H2O, CH4, N2O; */
/* band overlap handled within ck.c,   */
/* therefore only one component needs  */
/* to be considered here               */    

#define FU_COMPONENTS  1
#define FU_WGHT        1
#define FU_MAXINT     14


/* AVHRR parameterization by David Kratz             */
/* components: CO2, O3, O2, H2O, CH4, N2O, F11, F12; */
/* band overlap handled within ck.c,                 */
/* therefore only one component needs                */
/* to be considered here                             */    

#define AVHRR_KRATZ_COMPONENTS   1
#define AVHRR_KRATZ_WGHT         1
#define AVHRR_KRATZ_MAXINT      20



int kato_readtables (ck_struct *crs, char *path, char *photon_filename);
int fu_readtables (ck_struct *crs, char *path, char *photon_filename);
int avhrr_kratz_readtables (ck_struct *ck, char *path);
int crs_ck (ck_struct *ck, 
	    float *lambda, int n_lambda, 
	    float *temp, float *press, float *z, float **dens, int nlev,
	    float *mixing_ratio, int h2ocont,
	    char *path, int first, crs_ck_out_struct *crs_ck);

double fu_rayleigh (double pressure, double temperature, int iv);

#endif
